# The DipEncoder: Enforcing Multimodality in Autoencoders

Here, you can find the sourcecode of the 'The DipEncoder: Enforcing Multimodality in Autoencoders' paper.

## Preparation

An installation of python3 and a working C compiler are required!

Furthermore, following python packages must be installed:
- numpy
- scipy
- scikit-learn
- torch
- torchvision

We need to compile the dip.c file using a C compiler. For this following command needs to be executed (here we chose the gcc compiler):

On Windows: `gcc -fPIC -shared -std=c99 -o dip.dll dip.c`

On Linux: `gcc -fPIC -shared -o dip.so dip.c`

After this our code is ready to be executed.

## Data sets

The used data sets (Optdigits, USPS, HAR, Reuters10k, 20Newsgroups, MNIST, F-MNIST, K-MNIST) are integtated into our code and will be downloaded if necessary.

All data sets will be downloaded to: [HOME]/Downloads/DipEncoder_data/

If the automatic downloads do not work, the data sets can be downloaded here:
- Optdigits: https://archive.ics.uci.edu/ml/datasets/optical+recognition+of+handwritten+digits
- USPS: https://www.csie.ntu.edu.tw/~cjlin/libsvmtools/datasets/multiclass.html#usps
- HAR: https://archive.ics.uci.edu/ml/datasets/human+activity+recognition+using+smartphones
- Pendigits: https://archive.ics.uci.edu/ml/datasets/Pen-Based+Recognition+of+Handwritten+Digits
- Reuters: https://martin-thoma.com/nlp-reuters/
- 20Newsgroups: http://qwone.com/~jason/20Newsgroups/
- MNIST: http://yann.lecun.com/exdb/mnist/
- F-MNIST: https://github.com/zalandoresearch/fashion-mnist
- K-MNIST: https://github.com/rois-codh/kmnist
- Letterrecognition: https://archive.ics.uci.edu/ml/datasets/Letter+Recognition

## Execute

Everything is prepared in the main.py file.
Just uncomment the data set in the main() method and run the algorithm.

You can run the code by executing:

`python main.py`

or

`python3 main.py`

Alternatively, a python3 console can be opened and the following commands executed:

```python 
from dipencoder.dataloader import *
from dipencoder.dip_encoder import DipModuleClustering
from sklearn.metrics import normalized_mutual_info_score as nmi, adjusted_rand_score as ari

# Load data
data, gt_labels = load_usps()
# Get number of clusters
n_clusters= len(np.unique(gt_labels))
# Create DipEncoder
dipEncoder = DipModuleClustering(n_clusters)
dipEncoder.fit(data)
# Print results
print("NMI:", nmi(gt_labels, dipEncoder.labels_))
print("ARI:", ari(gt_labels, dipEncoder.labels_))
```
